
local addon = CreateFrame( "Frame", "hcRoleBot" )

--- Dispatches events to this addon by calling the method when the 
-- same name as the given event.
function addon:OnEvent( event, ... )
	if self[ event ] then
		self[ event ]( self, event, ... )
	end
end

--- Fired when the login process is complete.
-- This registers the events fired when critters should be summoned 
-- or dismissed.
function addon:PLAYER_LOGIN( event )
	if not IsLoggedIn( ) then
		return
	end
	
	self:UnregisterAllEvents( )
	
	self:RegisterEvent( "ACTIVE_TALENT_GROUP_CHANGED" )
	self:RegisterEvent( "PARTY_CONVERTED_TO_RAID" )
	self:RegisterEvent( "PARTY_MEMBERS_CHANGED" )
	self:RegisterEvent( "ROLE_POLL_BEGIN" )
end

local _ROLE_DAMAGE = "DAMAGER"
local _ROLE_HEAL = "HEALER"
local _ROLE_TANK = "TANK"

-- Checks the Feral talent tree for the presence of tanking talents.
-- @return "TANK" if tanking talents were found, "DAMAGER" otherwise.
local function _GetFeralDruidRole( )
	-- "Thick Hide" is the talent / spell to check
	-- spell id's = 16929, 16930, 16931
	-- talent = tab 2, index 11 (TODO check this)
	
	local _, _, _, _, rank, _, _, _, _, _ = GetTalentInfo( 2, 11, false, false )
	if rank > 0 then
		return _ROLE_TANK
	else
		return _ROLE_DAMAGE
	end
end

-- Defines the roles for particular classes and specs.
local _ROLES = {
	[ "DEATHKNIGHT" ] = { _ROLE_TANK, _ROLE_DAMAGE, _ROLE_DAMAGE, },
	[ "DRUID" ] = { _ROLE_DAMAGE, _GetFeralDruidRole( ), _ROLE_HEAL, },
	[ "HUNTER" ] = _ROLE_DAMAGE,
	[ "MAGE" ] = _ROLE_DAMAGE,
	[ "PALADIN" ] = { _ROLE_HEAL, _ROLE_TANK, _ROLE_DAMAGE, },
	[ "PRIEST" ] = { _ROLE_HEAL, _ROLE_HEAL, _ROLE_DAMAGE, },
	[ "ROGUE" ] = _ROLE_DAMAGE,
	[ "SHAMAN" ] = { _ROLE_DAMAGE, _ROLE_DAMAGE, _ROLE_HEAL, },
	[ "WARLOCK" ] = _ROLE_DAMAGE,
	[ "WARRIOR" ] = { _ROLE_DAMAGE, _ROLE_DAMAGE, _ROLE_TANK, },
}

-- Determines the best role for a particular class and talent spec.
-- @param class a class name.
-- @param spec the primary talent tree index.
-- @return one of "DAMAGER, "HEALER", or "TANK"
local function _GetSpecRole( class, spec )
	local roles = _ROLES[ class ]
	if type( roles ) == "table" then
		local role = roles[ spec ]
		if type( role ) == "function" then
			return role( )
		end
		return role
	end
	return roles
end

-- Determines the best role for the player, based on the player class 
-- and current talent spec.
local function _PlayerGetSpecRole( )
	local _, playerClass = UnitClass( "player" )
	local playerSpec = GetPrimaryTalentTree( false, false )
	return _GetSpecRole( playerClass, playerSpec )
end

-- Sets the player role based on the class and active spec. If the role 
-- is already set to the desired role, no change is made.
local function _PlayerSetSpecRole( )
	local role = _PlayerGetSpecRole( )
	local current = UnitGroupRolesAssigned( "player" )
	if role ~= current then
		UnitSetRole( "player", role )
	end
end

--- Fires when the player switches talent builds.
function addon:ACTIVE_TALENT_GROUP_CHANGED( event )
	_PlayerSetSpecRole( )
end

--- Fires when the player's party becomes a raid group.
function addon:PARTY_CONVERTED_TO_RAID( event )
	_PlayerSetSpecRole( )
end

--- Fires when group membership information changes.
-- This is used to check when the player joins some sort of group, or 
-- a role change is made for any group member.
function addon:PARTY_MEMBERS_CHANGED( event )
	_PlayerSetSpecRole( )
end

--- Fires when a role check is started for a group.
function addon:ROLE_POLL_BEGIN( event )
	_PlayerSetSpecRole( )
	StaticPopupSpecial_Hide( RolePollPopup )
end

-- Register the event handler and initial events.
addon:SetScript( "OnEvent", addon.OnEvent )
addon:RegisterEvent( "PLAYER_LOGIN" )
